/*
 * Copyright 2018-2019 NXP
 * All rights reserved.
 *
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */
#ifndef _APP_H_
#define _APP_H_

/*******************************************************************************
 * Definitions
 ******************************************************************************/
/*${macro:start}*/
#define EXAMPLE_FLEXSPI FLEXSPI
#define EXAMPLE_FLEXSPI_AMBA_BASE FlexSPI_AMBA_BASE
#define FLASH_SIZE 0x80000 /* 4096 x 64 x 2048 bytes */ // 0x4000 /* 128Mb/KByte */
#define FLASH_PAGE_SIZE 0x1000 //(4KB - see below)//256
#define SECTOR_SIZE 0x200 //(512B) //0x1000 /* 4K */
#define BLOCK_SIZE 0x42000
#define EXAMPLE_FLEXSPI_CLOCK kCLOCK_FlexSpi
#define EXAMPLE_BLOCK_TO_ERASE 0
#define EXAMPLE_PAGE_TO_PROGRAM_READ 0

//Toshiba NAND Flash sizes:
//	Total Size; 512MB (4Gbit)
//	Page: smallest addressable unit for read/write ops 				=== 4096 bytes (excluding ecc)
//	Block: 64 Pages=1 block (NAND erase operations @ single block)	=== 256K + 8K bytes
//	Sector: The 512 bytes unit in a page.							=== 512 bytes


#define CUSTOM_LUT_LENGTH 60


//Questions:
//1. how to decide order of indeces in LUT?															A: looks like it's a free for all. I write LUT, when i do a req for r/w - i specify idx.
//																										Major catch: if i want to work w/ boot - probably have to use the indexing specified in boot
//2. How to handle dummy bits?:  					 												A: appears to be a dummy command.
//3. Command arguments??? ie - What's the point of 24/32 bits of address - what about data bits?	A: # of clocks to allow for this cmd



#define NAND_CMD_LUT_SEQ_IDX_READ_FROM_CACHE 0
#define NAND_CMD_LUT_SEQ_IDX_READSTATUS 1
#define NAND_CMD_LUT_SEQ_IDX_WRITEENABLE 2
#define NAND_CMD_LUT_SEQ_IDX_BLOCKERASE 3
#define NAND_CMD_LUT_SEQ_IDX_PROGRAMLOAD 4

#define NAND_CMD_LUT_SEQ_IDX_READ_PAGE 5
#define NAND_CMD_LUT_SEQ_IDX_READ_ECC_STATUS 6
#define NAND_CMD_LUT_SEQ_IDX_PROGRAM_EXECUTE 7
#define NAND_CMD_LUT_SEQ_IDX_READ_FROM_CACHE_ODD 8

#define NAND_CMD_LUT_SEQ_IDX_PROGRAM_LOAD_ODD 9
#define NAND_CMD_LUT_SEQ_IDX_READID 10



/*${macro:end}*/

/*******************************************************************************
 * Prototypes
 ******************************************************************************/
/*${prototype:start}*/
void BOARD_InitHardware(void);

static inline void flexspi_clock_init()
{
#if defined(XIP_EXTERNAL_FLASH) && (XIP_EXTERNAL_FLASH == 1)
    /* Switch to PLL2 for XIP to avoid hardfault during re-initialize clock. */
    CLOCK_InitSysPfd(kCLOCK_Pfd2, 24);    /* Set PLL2 PFD2 clock 396MHZ. */
    CLOCK_SetMux(kCLOCK_FlexspiMux, 0x2); /* Choose PLL2 PFD2 clock as flexspi source clock. */
    CLOCK_SetDiv(kCLOCK_FlexspiDiv, 2);   /* flexspi clock 133M. */
#else
    const clock_usb_pll_config_t g_ccmConfigUsbPll = {.loopDivider = 0U};

    CLOCK_InitUsb1Pll(&g_ccmConfigUsbPll);
    CLOCK_InitUsb1Pfd(kCLOCK_Pfd0, 24);   /* Set PLL3 PFD0 clock 360MHZ. */
    CLOCK_SetMux(kCLOCK_FlexspiMux, 0x3); /* Choose PLL3 PFD0 clock as flexspi source clock. */
    CLOCK_SetDiv(kCLOCK_FlexspiDiv, 2);   /* flexspi clock 120M. */
#endif
}
/*${prototype:end}*/

#endif /* _APP_H_ */
